from datetime import datetime

from authlib.integrations.httpx_client import AsyncOAuth2Client
from pydantic import BaseModel, Field
from quart import Blueprint
from quart_schema import validate_response

from foxnouns import settings
from foxnouns.db import redis
from foxnouns.db.redis import discord_state_key
from foxnouns.models.user import SelfUserModel

bp = Blueprint("auth_v2", __name__)


def discord_oauth_client(*, state: str | None = None):
    """Returns a new Discord OAuth client with the given state.
    It is the caller's responsibility to check whether the client ID and client secret are set."""

    return AsyncOAuth2Client(
        settings.DISCORD_CLIENT_ID,
        settings.DISCORD_CLIENT_SECRET,
        state=state,
    )


class URLsResponse(BaseModel):
    discord: str | None = Field(default=None)
    google: str | None = Field(default=None)
    tumblr: str | None = Field(default=None)


@bp.post("/api/v2/auth/urls", host=settings.BASE_DOMAIN)
@validate_response(URLsResponse, 200)
async def urls():
    if settings.DISCORD_CLIENT_ID and settings.DISCORD_CLIENT_SECRET:
        client = discord_oauth_client()
        discord_url, state = client.create_authorization_url(
            "https://discord.com/api/oauth2/authorize"
        )

        await redis.set(
            discord_state_key(state), state, ex=1800
        )  # Expire after 30 minutes

    # TODO: return the other OAuth URLs
    return URLsResponse(discord=discord_url)


class OAuthCallbackRequest(BaseModel):
    callback_domain: str
    code: str
    state: str


class BaseCallbackResponse(BaseModel):
    """The base class for callback responses."""

    has_account: bool


class ExistingUserCallbackResponse(BaseCallbackResponse):
    """The class returned when a user already exists."""

    token: str
    user: SelfUserModel


class NewUserCallbackResponse(BaseCallbackResponse):
    """The class returned when the user is new and has to create an account."""

    remote_username: str
    ticket: str
    require_captcha: bool


class DeletedUserCallbackResponse(BaseCallbackResponse):
    """The class returned when the user has been deleted."""

    token: str
    user: SelfUserModel

    deleted_at: datetime
    self_delete: bool
    delete_reason: str | None = Field(default=None)
