from pydantic import Field
from quart import Blueprint, g
from quart_schema import validate_request, validate_response

from foxnouns import tasks
from foxnouns.db import Member
from foxnouns.db.aio import async_session
from foxnouns.db.util import user_from_ref
from foxnouns.decorators import require_auth
from foxnouns.exceptions import ErrorCode, NotFoundError
from foxnouns.models.member import FullMemberModel, MemberPatchModel
from foxnouns.settings import BASE_DOMAIN

bp = Blueprint("members_v2", __name__)


@bp.get("/api/v2/users/<user_ref>/members", host=BASE_DOMAIN)
@validate_response(list[FullMemberModel], 200)
async def get_members(user_ref: str):
    async with async_session() as session:
        user = await user_from_ref(session, user_ref)
        if not user:
            raise NotFoundError("User not found", type=ErrorCode.UserNotFound)

        return [FullMemberModel.model_validate(m) for m in user.members]


class MemberCreateModel(MemberPatchModel):
    name: str = Field(
        min_length=1,
        max_length=100,
        pattern=r"^[^@\?!#\/\\\[\]\"\{\}'$%&()+<=>^|~`,\*]{1,100}$",
    )


@bp.post("/api/v2/members", host=BASE_DOMAIN)
@require_auth(scope="member.create")
@validate_request(MemberCreateModel)
@validate_response(FullMemberModel, 200)
async def create_member(data: MemberCreateModel):
    async with async_session() as session:
        member = Member(
            user_id=g.user.id,
            name=data.name,
            bio=data.bio,
            names=[e.model_dump() for e in data.names],
            pronouns=[e.model_dump() for e in data.pronouns],
            fields=[e.model_dump() for e in data.fields],
        )

        session.add(member)
        await session.commit()
        # This has to be fetched before we can pass the model to Pydantic.
        # In a normal SELECT this is automatically fetched, but because we just created the object,
        # we have to do it manually.
        await member.awaitable_attrs.user

        if data.avatar:
            tasks.process_member_avatar.delay(member.id, data.avatar)

        return FullMemberModel.model_validate(member)
